-- Project P8  - CSD http://digsys.upc.edu                          

-- Component Counter_MMSS to count up to 59:59 and down to 00:00  
-- Strategy plan C2: use 2 Counter_BCD_mod60 in a chain

-- TCH is '1' only when CE = 1 and UP and 59:59 or when DOWN and 00:00  

-- Translation schematic (Fig. 2) at:
-- https://digsys.upc.edu/csd/P08/P8.html#Counter
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Counter_MMSS IS
	PORT (
		UD_L, CE, CD, CLK	: IN STD_LOGIC;
		DMT, DMU			: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	  
		DST, DSU			: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		TCH					: OUT STD_LOGIC
	 );
END Counter_MMSS;

ARCHITECTURE hierarchical_structure OF Counter_MMSS IS
	
-- The elemental components to be used:
 	COMPONENT Counter_BCD_mod60 IS  
	    PORT (
	        UD_L, CE, CD, CLK   : IN    STD_LOGIC;
	        D_Tens, D_Units     : OUT   STD_LOGIC_VECTOR (3 DOWNTO 0);
	        TC60                : OUT   STD_LOGIC
	     );
	END COMPONENT;

-- Now the signals (wires and cables) to build the schematic 

	SIGNAL	TC60S, TC60M	: STD_LOGIC;

BEGIN

-- Instantiation of the 2 counter_mod60 
	Chip1	: Counter_BCD_mod60   -- The seconds counter 
		PORT MAP (
	-- from component name 	=> to signal or port name		
			CLK			=>	CLK,
			CD			=>	CD,				
			CE			=>	CE,    -- The general CE
			UD_L		=> 	UD_L,
			D_Tens		=>	DST,  -- Digit_Seconds_Tens
			D_Units 	=>	DSU,  -- Digit_Seconds_Units
			TC60		=>	TC60S
		);

	Chip2	: Counter_BCD_mod60   -- The minutes counter 
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		    =>	CLK,
			CD			=>	CD,				
			CE			=>	TC60S,  	-- The CE of the minutes  is chained
										-- to the TC60S
			UD_L		=>	UD_L,
			D_Tens	    =>	DMT,  			-- Digit_Seconds_Tens
			D_Units 	=>	DMU,  		-- Digit_Seconds_Units
			TC60		=>	TC60M
		);
		
						
-- Outputs 		
		TCH		<= TC60S and TC60M;
		
END hierarchical_structure;
